@@ -35,17 +35,17 @@ def new(output_dir):
         return
 
     if not os.path.exists(output_dir):
-        log.info('Creating project directory: %s', output_dir)
+        log.info(f'Creating project directory: {output_dir}')
         os.mkdir(output_dir)
 
-    log.info('Writing config file: %s', config_path)
+    log.info(f'Writing config file: {config_path}')
     with open(config_path, 'w', encoding='utf-8') as f:
         f.write(config_text)
 
     if os.path.exists(index_path):
         return
 
-    log.info('Writing initial docs: %s', index_path)
+    log.info(f'Writing initial docs: {index_path}')
     if not os.path.exists(docs_dir):
         os.mkdir(docs_dir)
     with open(index_path, 'w', encoding='utf-8') as f:
