@@ -1,6 +1,7 @@
 import os
-from collections import Sequence, namedtuple
-from urllib.parse import urlparse
+from collections import namedtuple
+from collections.abc import Sequence
+from urllib.parse import urlsplit, urlunsplit
 import ipaddress
 import markdown
 
@@ -39,7 +40,7 @@ class BaseConfigOption:
     def post_validation(self, config, key_name):
         """
         After all options have passed validation, perform a post-validation
-        process to do any additional changes dependant on other config values.
+        process to do any additional changes dependent on other config values.
         The post-validation process method should be implemented by subclasses.
         """
 
@@ -71,7 +72,7 @@ class ConfigItems(BaseConfigOption):
         self.required = kwargs.get('required', False)
 
     def __repr__(self):
-        return '{}: {}'.format(self.__class__.__name__, self.item_config)
+        return f'{self.__class__.__name__}: {self.item_config}'
 
     def run_validation(self, value):
         if value is None:
@@ -81,12 +82,10 @@ class ConfigItems(BaseConfigOption):
                 return ()
 
         if not isinstance(value, Sequence):
-            raise ValidationError('Expected a sequence of mappings, but a %s '
-                                  'was given.' % type(value))
-        result = []
-        for item in value:
-            result.append(self.item_config.validate(item))
-        return result
+            raise ValidationError(f'Expected a sequence of mappings, but a '
+                                  f'{type(value)} was given.')
+
+        return [self.item_config.validate(item) for item in value]
 
 
 class OptionallyRequired(BaseConfigOption):
@@ -140,8 +139,7 @@ class Type(OptionallyRequired):
     def run_validation(self, value):
 
         if not isinstance(value, self._type):
-            msg = ("Expected type: {} but received: {}"
-                   .format(self._type, type(value)))
+            msg = f"Expected type: {self._type} but received: {type(value)}"
         elif self.length is not None and len(value) != self.length:
             msg = ("Expected type: {0} with length {2} but received: {1} with "
                    "length {3}").format(self._type, value, self.length,
@@ -166,14 +164,13 @@ class Choice(OptionallyRequired):
             length = 0
 
         if not length or isinstance(choices, str):
-            raise ValueError('Expected iterable of choices, got {}', choices)
+            raise ValueError(f'Expected iterable of choices, got {choices}')
 
         self.choices = choices
 
     def run_validation(self, value):
         if value not in self.choices:
-            msg = ("Expected one of: {} but received: {}"
-                   .format(self.choices, value))
+            msg = f"Expected one of: {self.choices} but received: {value}"
         else:
             return value
 
@@ -181,43 +178,63 @@ class Choice(OptionallyRequired):
 
 
 class Deprecated(BaseConfigOption):
+    """
+    Deprecated Config Option
+    Raises a warning as the option is deprecated. Uses `message` for the
+    warning. If `move_to` is set to the name of a new config option, the value
+    is moved to the new option on pre_validation. If `option_type` is set to a
+    ConfigOption instance, then the value is validated against that type.
+    """
 
-    def __init__(self, moved_to=None):
+    def __init__(self, moved_to=None, message='', option_type=None):
         super().__init__()
         self.default = None
         self.moved_to = moved_to
+        self.message = message or (
+            'The configuration option {} has been deprecated and '
+            'will be removed in a future release of Docums.'
+        )
+        self.option = option_type or BaseConfigOption()
+        self.warnings = self.option.warnings
 
     def pre_validation(self, config, key_name):
+        self.option.pre_validation(config, key_name)
 
-        if config.get(key_name) is None or self.moved_to is None:
-            return
+        if config.get(key_name) is not None:
+            self.warnings.append(self.message.format(key_name))
 
-        warning = ('The configuration option {} has been deprecated and '
-                   'will be removed in a future release of Docums.'
-                   ''.format(key_name))
-        self.warnings.append(warning)
+            if self.moved_to is not None:
+                if '.' not in self.moved_to:
+                    target = config
+                    target_key = self.moved_to
+                else:
+                    move_to, target_key = self.moved_to.rsplit('.', 1)
 
-        if '.' not in self.moved_to:
-            target = config
-            target_key = self.moved_to
-        else:
-            move_to, target_key = self.moved_to.rsplit('.', 1)
+                    target = config
+                    for key in move_to.split('.'):
+                        target = target.setdefault(key, {})
 
-            target = config
-            for key in move_to.split('.'):
-                target = target.setdefault(key, {})
+                        if not isinstance(target, dict):
+                            # We can't move it for the user
+                            return
 
-                if not isinstance(target, dict):
-                    # We can't move it for the user
-                    return
+                target[target_key] = config.pop(key_name)
 
-        target[target_key] = config.pop(key_name)
+    def validate(self, value):
+        return self.option.validate(value)
+
+    def post_validation(self, config, key_name):
+        self.option.post_validation(config, key_name)
+
+    def reset_warnings(self):
+        self.option.reset_warnings()
+        self.warnings = self.option.warnings
 
 
 class IpAddress(OptionallyRequired):
     """
     IpAddress Config Option
-    Validate that an IP address is in an apprioriate format
+    Validate that an IP address is in an appropriate format
     """
 
     def run_validation(self, value):
@@ -236,11 +253,11 @@ class IpAddress(OptionallyRequired):
         try:
             port = int(port)
         except Exception:
-            raise ValidationError("'{}' is not a valid port".format(port))
+            raise ValidationError(f"'{port}' is not a valid port")
 
         class Address(namedtuple('Address', 'host port')):
             def __str__(self):
-                return '{}:{}'.format(self.host, self.port)
+                return f'{self.host}:{self.port}'
 
         return Address(host, port)
 
@@ -249,7 +266,7 @@ class IpAddress(OptionallyRequired):
         if key_name == 'dev_addr' and host in ['0.0.0.0', '::']:
             self.warnings.append(
                 ("The use of the IP address '{}' suggests a production environment "
-                 "or the use of a proxy to connect to the Ddocumsdocumsocums server. However, "
+                 "or the use of a proxy to connect to the Docums server. However, "
                  "the Docums' server is intended for local development purposes only. "
                  "Please use a third party production-ready server instead.").format(host)
             )
@@ -261,7 +278,8 @@ class URL(OptionallyRequired):
     Validate a URL by requiring a scheme is present.
     """
 
-    def __init__(self, default='', required=False):
+    def __init__(self, default='', required=False, is_dir=False):
+        self.is_dir = is_dir
         super().__init__(default, required)
 
     def run_validation(self, value):
@@ -269,12 +287,14 @@ class URL(OptionallyRequired):
             return value
 
         try:
-            parsed_url = urlparse(value)
+            parsed_url = urlsplit(value)
         except (AttributeError, TypeError):
             raise ValidationError("Unable to parse the URL.")
 
-        if parsed_url.scheme:
-            return value
+        if parsed_url.scheme and parsed_url.netloc:
+            if self.is_dir and not parsed_url.path.endswith('/'):
+                parsed_url = parsed_url._replace(path=f'{parsed_url.path}/')
+            return urlunsplit(parsed_url)
 
         raise ValidationError(
             "The URL isn't valid, it should include the http:// (scheme)")
@@ -288,7 +308,7 @@ class RepoURL(URL):
     """
 
     def post_validation(self, config, key_name):
-        repo_host = urlparse(config['repo_url']).netloc.lower()
+        repo_host = urlsplit(config['repo_url']).netloc.lower()
         edit_uri = config.get('edit_uri')
 
         # derive repo_name from repo_url if unset
@@ -314,7 +334,7 @@ class RepoURL(URL):
         # ensure a well-formed edit_uri
         if edit_uri:
             if not edit_uri.startswith(('?', '#')) \
-                and not config['repo_url'].endswith('/'):
+                    and not config['repo_url'].endswith('/'):
                 config['repo_url'] += '/'
             if not edit_uri.endswith('/'):
                 edit_uri += '/'
@@ -339,8 +359,7 @@ class FilesystemObject(Type):
         if self.config_dir and not os.path.isabs(value):
             value = os.path.join(self.config_dir, value)
         if self.exists and not self.existence_test(value):
-            raise ValidationError("The path {path} isn't an existing {name}.".
-                                  format(path=value, name=self.name))
+            raise ValidationError(f"The path {value} isn't an existing {self.name}.")
         value = os.path.abspath(value)
         assert isinstance(value, str)
         return value
@@ -429,13 +448,13 @@ class Theme(BaseConfigOption):
                     return value
 
                 raise ValidationError(
-                    "Unrecognised theme name: '{}'. The available installed themes "
-                    "are: {}".format(value['name'], ', '.join(themes))
+                    f"Unrecognised theme name: '{value['name']}'. "
+                    f"The available installed themes are: {', '.join(themes)}"
                 )
 
             raise ValidationError("No theme name set.")
 
-        raise ValidationError('Invalid type "{}". Expected a string or key/value pairs.'.format(type(value)))
+        raise ValidationError(f'Invalid type "{type(value)}". Expected a string or key/value pairs.')
 
     def post_validation(self, config, key_name):
         theme_config = config[key_name]
@@ -452,6 +471,9 @@ class Theme(BaseConfigOption):
             raise ValidationError("The path set in {name}.custom_dir ('{path}') does not exist.".
                                   format(path=theme_config['custom_dir'], name=key_name))
 
+        if 'locale' in theme_config and not isinstance(theme_config['locale'], str):
+            raise ValidationError("'{name}.locale' must be a string.".format(name=theme_config['name']))
+
         config[key_name] = theme.Theme(**theme_config)
 
 
@@ -468,8 +490,7 @@ class Nav(OptionallyRequired):
     def run_validation(self, value):
 
         if not isinstance(value, list):
-            raise ValidationError(
-                "Expected a list, got {}".format(type(value)))
+            raise ValidationError(f"Expected a list, got {type(value)}")
 
         if len(value) == 0:
             return
@@ -506,38 +527,47 @@ class Private(OptionallyRequired):
 class MarkdownExtensions(OptionallyRequired):
     """
     Markdown Extensions Config Option
-    A list of extensions. If a list item contains extension configs,
-    those are set on the private  setting passed to `configkey`. The
-    `builtins` keyword accepts a list of extensions which cannot be
-    overriden by the user. However, builtins can be duplicated to define
-    config options for them if desired.
-    """
+    A list or dict of extensions. Each list item may contain either a string or a one item dict.
+    A string must be a valid Markdown extension name with no config options defined. The key of
+    a dict item must be a valid Markdown extension name and the value must be a dict of config
+    options for that extension. Extension configs are set on the private setting passed to
+    `configkey`. The `builtins` keyword accepts a list of extensions which cannot be overridden by
+    the user. However, builtins can be duplicated to define config options for them if desired. """
     def __init__(self, builtins=None, configkey='mdx_configs', **kwargs):
         super().__init__(**kwargs)
         self.builtins = builtins or []
         self.configkey = configkey
         self.configdata = {}
 
+    def validate_ext_cfg(self, ext, cfg):
+        if not isinstance(ext, str):
+            raise ValidationError(f"'{ext}' is not a valid Markdown Extension name.")
+        if not cfg:
+            return
+        if not isinstance(cfg, dict):
+            raise ValidationError(f"Invalid config options for Markdown Extension '{ext}'.")
+        self.configdata[ext] = cfg
+
     def run_validation(self, value):
-        if not isinstance(value, (list, tuple)):
+        if not isinstance(value, (list, tuple, dict)):
             raise ValidationError('Invalid Markdown Extensions configuration')
         extensions = []
-        for item in value:
-            if isinstance(item, dict):
-                if len(item) > 1:
-                    raise ValidationError('Invalid Markdown Extensions configuration')
-                ext, cfg = item.popitem()
+        if isinstance(value, dict):
+            for ext, cfg in value.items():
+                self.validate_ext_cfg(ext, cfg)
                 extensions.append(ext)
-                if cfg is None:
-                    continue
-                if not isinstance(cfg, dict):
-                    raise ValidationError('Invalid config options for Markdown '
-                                          "Extension '{}'.".format(ext))
-                self.configdata[ext] = cfg
-            elif isinstance(item, str):
-                extensions.append(item)
-            else:
-                raise ValidationError('Invalid Markdown Extensions configuration')
+        else:
+            for item in value:
+                if isinstance(item, dict):
+                    if len(item) > 1:
+                        raise ValidationError('Invalid Markdown Extensions configuration')
+                    ext, cfg = item.popitem()
+                    self.validate_ext_cfg(ext, cfg)
+                    extensions.append(ext)
+                elif isinstance(item, str):
+                    extensions.append(item)
+                else:
+                    raise ValidationError('Invalid Markdown Extensions configuration')
 
         extensions = utils.reduce_list(self.builtins + extensions)
 
@@ -556,7 +586,7 @@ class MarkdownExtensions(OptionallyRequired):
 class Plugins(OptionallyRequired):
     """
     Plugins config option.
-    A list of plugins. If a plugin defines config options those are used when
+    A list or dict of plugins. If a plugin defines config options those are used when
     initializing the plugin class.
     """
 
@@ -569,32 +599,33 @@ class Plugins(OptionallyRequired):
         self.config_file_path = config.config_file_path
 
     def run_validation(self, value):
-        if not isinstance(value, (list, tuple)):
-            raise ValidationError('Invalid Plugins configuration. Expected a list of plugins')
+        if not isinstance(value, (list, tuple, dict)):
+            raise ValidationError('Invalid Plugins configuration. Expected a list or dict.')
         plgins = plugins.PluginCollection()
-        for item in value:
-            if isinstance(item, dict):
-                if len(item) > 1:
-                    raise ValidationError('Invalid Plugins configuration')
-                name, cfg = item.popitem()
-                cfg = cfg or {}  # Users may define a null (None) config
-                if not isinstance(cfg, dict):
-                    raise ValidationError('Invalid config options for '
-                                          'the "{}" plugin.'.format(name))
-                item = name
-            else:
-                cfg = {}
-
-            if not isinstance(item, str):
-                raise ValidationError('Invalid Plugins configuration')
-
-            plgins[item] = self.load_plugin(item, cfg)
-
+        if isinstance(value, dict):
+            for name, cfg in value.items():
+                plgins[name] = self.load_plugin(name, cfg)
+        else:
+            for item in value:
+                if isinstance(item, dict):
+                    if len(item) > 1:
+                        raise ValidationError('Invalid Plugins configuration')
+                    name, cfg = item.popitem()
+                    item = name
+                else:
+                    cfg = {}
+                plgins[item] = self.load_plugin(item, cfg)
         return plgins
 
     def load_plugin(self, name, config):
+        if not isinstance(name, str):
+            raise ValidationError(f"'{name}' is not a valid plugin name.")
         if name not in self.installed_plugins:
-            raise ValidationError('The "{}" plugin is not installed'.format(name))
+            raise ValidationError(f'The "{name}" plugin is not installed')
+
+        config = config or {}  # Users may define a null (None) config
+        if not isinstance(config, dict):
+            raise ValidationError(f"Invalid config options for the '{name}' plugin.")
 
         Plugin = self.installed_plugins[name].load()
 
@@ -607,7 +638,7 @@ class Plugins(OptionallyRequired):
         errors, warnings = plugin.load_config(config, self.config_file_path)
         self.warnings.extend(warnings)
         errors_message = '\n'.join(
-            "Plugin value: '{}'. Error: {}".format(x, y)
+            f"Plugin '{name}' value: '{x}'. Error: {y}"
             for x, y in errors
         )
         if errors_message:
