@@ -1,11 +1,10 @@
 import logging
-from urllib.parse import urlparse
+from urllib.parse import urlsplit
 
 from docums.structure.pages import Page
-from docums.utils import nest_paths, warning_filter
+from docums.utils import nest_paths
 
 log = logging.getLogger(__name__)
-log.addFilter(warning_filter)
 
 
 class Navigation:
@@ -42,7 +41,7 @@ class Section:
         self.is_link = False
 
     def __repr__(self):
-        return "Section(title='{}')".format(self.title)
+        return f"Section(title='{self.title}')"
 
     def _get_active(self):
         """ Return active status of section. """
@@ -83,8 +82,8 @@ class Link:
         self.is_link = True
 
     def __repr__(self):
-        title = "'{}'".format(self.title) if (self.title is not None) else '[blank]'
-        return "Link(title={}, url='{}')".format(title, self.url)
+        title = f"'{self.title}'" if (self.title is not None) else '[blank]'
+        return f"Link(title={title}, url='{self.url}')"
 
     @property
     def ancestors(self):
@@ -119,13 +118,13 @@ def get_navigation(files, config):
         )
         # Any documentation files not found in the nav should still have an associated page, so we
         # create them here. The Page object will automatically be assigned to `file.page` during
-        # its creation (and this is the only way in which these page objects are accessable).
+        # its creation (and this is the only way in which these page objects are accessible).
         for file in missing_from_config:
             Page(None, file, config)
 
     links = _get_by_type(items, Link)
     for link in links:
-        scheme, netloc, path, params, query, fragment = urlparse(link.url)
+        scheme, netloc, path, query, fragment = urlsplit(link.url)
         if scheme or netloc:
             log.debug(
                 "An external link to '{}' is included in "
@@ -172,7 +171,7 @@ def _get_by_type(nav, T):
     for item in nav:
         if isinstance(item, T):
             ret.append(item)
-        elif item.children:
+        if item.children:
             ret.extend(_get_by_type(item.children, T))
     return ret
 
