@@ -5,9 +5,9 @@ import logging
 from docums import utils
 from docums.utils import filters
 from docums.config.base import ValidationError
+from docums import localization
 
 log = logging.getLogger(__name__)
-log.addFilter(utils.warning_filter)
 
 
 class Theme:
@@ -28,7 +28,7 @@ class Theme:
 
     def __init__(self, name=None, **user_config):
         self.name = name
-        self._vars = {}
+        self._vars = {'locale': 'en'}
 
         # Docums provided static templates are always included
         package_dir = os.path.abspath(os.path.dirname(__file__))
@@ -51,10 +51,13 @@ class Theme:
         self.static_templates.update(user_config.pop('static_templates', []))
         self._vars.update(user_config)
 
+        # Validate locale and convert to Locale object
+        self._vars['locale'] = localization.parse_locale(self._vars['locale'])
+
     def __repr__(self):
         return "{}(name='{}', dirs={}, static_templates={}, {})".format(
             self.__class__.__name__, self.name, self.dirs, list(self.static_templates),
-            ', '.join('{}={}'.format(k, repr(v)) for k, v in self._vars.items())
+            ', '.join(f'{k}={v!r}' for k, v in self._vars.items())
         )
 
     def __getitem__(self, key):
@@ -84,19 +87,19 @@ class Theme:
         except OSError as e:
             log.debug(e)
             raise ValidationError(
-                "The theme '{}' does not appear to have a configuration file. "
-                "Please upgrade to a current version of the theme.".format(name)
+                f"The theme '{name}' does not appear to have a configuration file. "
+                f"Please upgrade to a current version of the theme."
             )
 
-        log.debug("Loaded theme configuration for '%s' from '%s': %s", name, file_path, theme_config)
+        log.debug(f"Loaded theme configuration for '{name}' from '{file_path}': {theme_config}")
 
         parent_theme = theme_config.pop('extends', None)
         if parent_theme:
             themes = utils.get_theme_names()
             if parent_theme not in themes:
                 raise ValidationError(
-                    "The theme '{}' inherits from '{}', which does not appear to be installed. "
-                    "The available installed themes are: {}".format(name, parent_theme, ', '.join(themes))
+                    f"The theme '{name}' inherits from '{parent_theme}', which does not appear to be installed. "
+                    f"The available installed themes are: {', '.join(themes)}"
                 )
             self._load_theme_config(parent_theme)
 
@@ -107,7 +110,9 @@ class Theme:
         """ Return a Jinja environment for the theme. """
 
         loader = jinja2.FileSystemLoader(self.dirs)
-        env = jinja2.Environment(loader=loader)
+        # No autoreload because editing a template in the middle of a build is not useful.
+        env = jinja2.Environment(loader=loader, auto_reload=False)
         env.filters['tojson'] = filters.tojson
         env.filters['url'] = filters.url_filter
+        localization.install_translations(env, self._vars['locale'], self.dirs)
         return env
