@@ -1,6 +1,5 @@
 """
 Standalone file utils.
-
 Nothing in this module should have an knowledge of config or the layout
 and structure of the site and pages in the site.
 """
@@ -8,14 +7,18 @@ and structure of the site and pages in the site.
 
 import logging
 import os
-import pkg_resources
 import shutil
 import re
 import yaml
 import fnmatch
 import posixpath
+import functools
+import importlib_metadata
+from collections import defaultdict
 from datetime import datetime, timezone
-from urllib.parse import urlparse
+from urllib.parse import urlsplit
+from yaml_env_tag import construct_env_tag
+from mergedeep import merge
 
 from docums import exceptions
 
@@ -30,20 +33,8 @@ markdown_extensions = [
 ]
 
 
-def yaml_load(source, loader=yaml.Loader):
-    """
-    Wrap PyYaml's loader so we can extend it to suit our needs.
-
-    Load all strings as unicode.
-    https://stackoverflow.com/a/2967461/3609487
-    """
-
-    def construct_yaml_str(self, node):
-        """
-        Override the default string handling function to always return
-        unicode objects.
-        """
-        return self.construct_scalar(node)
+def get_yaml_loader(loader=yaml.Loader):
+    """ Wrap PyYaml's loader so we can extend it to suit our needs. """
 
     class Loader(loader):
         """
@@ -51,22 +42,28 @@ def yaml_load(source, loader=yaml.Loader):
         global loader unaltered.
         """
 
-    # Attach our unicode constructor to our custom loader ensuring all strings
-    # will be unicode on translation.
-    Loader.add_constructor('tag:yaml.org,2002:str', construct_yaml_str)
+    # Attach Environment Variable constructor.
+    # See https://github.com/waylan/pyyaml-env-tag
+    Loader.add_constructor('!ENV', construct_env_tag)
 
-    try:
-        return yaml.load(source, Loader)
-    finally:
-        # TODO: Remove this when external calls are properly cleaning up file
-        # objects. Some docums internal calls, sometimes in test lib, will
-        # load configs with a file object but never close it.  On some
-        # systems, if a delete action is performed on that file without Python
-        # closing that object, there will be an access error. This will
-        # process the file and close it as there should be no more use for the
-        # file once we process the yaml content.
-        if hasattr(source, 'close'):
-            source.close()
+    return Loader
+
+
+def yaml_load(source, loader=None):
+    """ Return dict of source YAML file using loader, recursively deep merging inherited parent. """
+    Loader = loader or get_yaml_loader()
+    result = yaml.load(source, Loader=Loader)
+    if result is not None and 'INHERIT' in result:
+        relpath = result.pop('INHERIT')
+        abspath = os.path.normpath(os.path.join(os.path.dirname(source.name), relpath))
+        if not os.path.exists(abspath):
+            raise exceptions.ConfigurationError(
+                f"Inherited config file '{relpath}' does not exist at '{abspath}'.")
+        log.debug(f"Loading inherited configuration file: {abspath}")
+        with open(abspath, 'rb') as fd:
+            parent = yaml_load(fd, Loader)
+        result = merge(parent, result)
+    return result
 
 
 def modified_time(file_path):
@@ -125,12 +122,10 @@ def reduce_list(data_set):
 def copy_file(source_path, output_path):
     """
     Copy source_path to output_path, making sure any parent directories exist.
-
     The output_path may be a directory.
     """
     output_dir = os.path.dirname(output_path)
-    if not os.path.exists(output_dir):
-        os.makedirs(output_dir)
+    os.makedirs(output_dir, exist_ok=True)
     if os.path.isdir(output_path):
         output_path = os.path.join(output_path, os.path.basename(source_path))
     shutil.copyfile(source_path, output_path)
@@ -141,8 +136,7 @@ def write_file(content, output_path):
     Write content to output_path, making sure any parent directories exist.
     """
     output_dir = os.path.dirname(output_path)
-    if not os.path.exists(output_dir):
-        os.makedirs(output_dir)
+    os.makedirs(output_dir, exist_ok=True)
     with open(output_path, 'wb') as f:
         f.write(content)
 
@@ -171,7 +165,6 @@ def clean_directory(directory):
 def get_html_path(path):
     """
     Map a source file path to an output html path.
-
     Paths like 'index.md' will be converted to 'index.html'
     Paths like 'about.md' will be converted to 'about/index.html'
     Paths like 'api-guide/core.md' will be converted to 'api-guide/core/index.html'
@@ -185,11 +178,9 @@ def get_html_path(path):
 def get_url_path(path, use_directory_urls=True):
     """
     Map a source file path to an output html path.
-
     Paths like 'index.md' will be converted to '/'
     Paths like 'about.md' will be converted to '/about/'
     Paths like 'api-guide/core.md' will be converted to '/api-guide/core/'
-
     If `use_directory_urls` is `False`, returned URLs will include the a trailing
     `index.html` rather than just returning the directory path.
     """
@@ -203,10 +194,9 @@ def get_url_path(path, use_directory_urls=True):
 def is_markdown_file(path):
     """
     Return True if the given file path is a Markdown file.
-
     https://superuser.com/questions/249436/file-extension-for-markdown-files
     """
-    return any(fnmatch.fnmatch(path.lower(), '*{}'.format(x)) for x in markdown_extensions)
+    return any(fnmatch.fnmatch(path.lower(), f'*{x}') for x in markdown_extensions)
 
 
 def is_html_file(path):
@@ -242,43 +232,66 @@ def is_error_template(path):
     return bool(_ERROR_TEMPLATE_RE.match(path))
 
 
+@functools.lru_cache(maxsize=None)
+def _norm_parts(path):
+    if not path.startswith('/'):
+        path = '/' + path
+    path = posixpath.normpath(path)[1:]
+    return path.split('/') if path else []
+
+
 def get_relative_url(url, other):
     """
     Return given url relative to other.
+    Both are operated as slash-separated paths, similarly to the 'path' part of a URL.
+    The last component of `other` is skipped if it contains a dot (considered a file).
+    Actual URLs (with schemas etc.) aren't supported. The leading slash is ignored.
+    Paths are normalized ('..' works as parent directory), but going higher than the
+    root has no effect ('foo/../../bar' ends up just as 'bar').
     """
-    if other != '.':
-        # Remove filename from other url if it has one.
-        parts = posixpath.split(other)
-        other = parts[0] if '.' in parts[1] else other
-    relurl = posixpath.relpath(url, other)
+    # Remove filename from other url if it has one.
+    dirname, _, basename = other.rpartition('/')
+    if '.' in basename:
+        other = dirname
+
+    other_parts = _norm_parts(other)
+    dest_parts = _norm_parts(url)
+    common = 0
+    for a, b in zip(other_parts, dest_parts):
+        if a != b:
+            break
+        common += 1
+
+    rel_parts = ['..'] * (len(other_parts) - common) + dest_parts[common:]
+    relurl = '/'.join(rel_parts) or '.'
     return relurl + '/' if url.endswith('/') else relurl
 
 
 def normalize_url(path, page=None, base=''):
     """ Return a URL relative to the given page or using the base. """
-    path = path_to_url(path or '.')
-    # Allow links to be fully qualified URL's
-    parsed = urlparse(path)
-    if parsed.scheme or parsed.netloc or path.startswith(('/', '#')):
+    path, is_abs = _get_norm_url(path)
+    if is_abs:
         return path
-
-    # We must be looking at a local path.
     if page is not None:
         return get_relative_url(path, page.url)
-    else:
-        return posixpath.join(base, path)
+    return posixpath.join(base, path)
+
+
+@functools.lru_cache(maxsize=None)
+def _get_norm_url(path):
+    path = path_to_url(path or '.')
+    # Allow links to be fully qualified URLs
+    parsed = urlsplit(path)
+    if parsed.scheme or parsed.netloc or path.startswith(('/', '#')):
+        return path, True
+    return path, False
 
 
 def create_media_urls(path_list, page=None, base=''):
     """
     Return a list of URLs relative to the given page or using the base.
     """
-    urls = []
-
-    for path in path_list:
-        urls.append(normalize_url(path, page, base))
-
-    return urls
+    return [normalize_url(path, page, base) for path in path_list]
 
 
 def path_to_url(path):
@@ -295,23 +308,24 @@ def get_theme_dir(name):
 
 
 def get_themes():
-    """ Return a dict of all installed themes as (name, entry point) pairs. """
+    """ Return a dict of all installed themes as {name: EntryPoint}. """
 
     themes = {}
-    builtins = pkg_resources.get_entry_map(dist='docums', group='docums.themes')
+    eps = set(importlib_metadata.entry_points(group='docums.themes'))
+    builtins = {ep.name for ep in eps if ep.dist.name == 'docums'}
 
-    for theme in pkg_resources.iter_entry_points(group='docums.themes'):
+    for theme in eps:
 
-        if theme.name in builtins and theme.dist.key != 'docums':
+        if theme.name in builtins and theme.dist.name != 'docums':
             raise exceptions.ConfigurationError(
-                "The theme {} is a builtin theme but {} provides a theme "
-                "with the same name".format(theme.name, theme.dist.key))
-
+                f"The theme '{theme.name}' is a builtin theme but the package '{theme.dist.name}' "
+                "attempts to provide a theme with the same name."
+            )
         elif theme.name in themes:
-            multiple_packages = [themes[theme.name].dist.key, theme.dist.key]
-            log.warning("The theme %s is provided by the Python packages "
-                        "'%s'. The one in %s will be used.",
-                        theme.name, ','.join(multiple_packages), theme.dist.key)
+            log.warning(
+                f"A theme named '{theme.name}' is provided by the Python packages '{theme.dist.name}' "
+                f"and '{themes[theme.name].dist.name}'. The one in '{theme.dist.name}' will be used."
+            )
 
         themes[theme.name] = theme
 
@@ -400,15 +414,25 @@ def nest_paths(paths):
     return nested
 
 
-class WarningFilter(logging.Filter):
-    """ Counts all WARNING level log messages. """
-    count = 0
+class CountHandler(logging.NullHandler):
+    """ Counts all logged messages >= level. """
 
-    def filter(self, record):
-        if record.levelno == logging.WARNING:
-            self.count += 1
-        return True
+    def __init__(self, **kwargs):
+        self.counts = defaultdict(int)
+        super().__init__(**kwargs)
+
+    def handle(self, record):
+        rv = self.filter(record)
+        if rv:
+            # Use levelno for keys so they can be sorted later
+            self.counts[record.levelno] += 1
+        return rv
+
+    def get_counts(self):
+        return [(logging.getLevelName(k), v) for k, v in sorted(self.counts.items(), reverse=True)]
 
 
-# A global instance to use throughout package
-warning_filter = WarningFilter()
+# For backward compatibility as some plugins import it.
+# It is no longer necessary as all messages on the
+# `docums` logger get counted automatically.
+warning_filter = logging.Filter()
